/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WF0PS_H
#define WF0PS_H

#include "flow.h"
#include "wObserver.h"
#include <Eigen/Dense>

namespace flow
{

/**
 * @brief Scalar functions depending on spatial position
 */
class FLOW_API F0Ps : public fwk::Observer
{
public:
    F0Ps() : Observer() {}
    virtual ~F0Ps() {}

    virtual double eval(Eigen::Vector3d const &pos) const = 0;
};

/**
 * @brief Freestream potential
 */
class FLOW_API F0PsPhiInf : public F0Ps
{
    int nDim;     ///< number of dimensions
    double alpha; ///< angle of attack
    double beta;  ///< angle of sideslip

public:
    F0PsPhiInf(int _nDim, double _alpha, double _beta = 0);
    virtual void update(double _alpha) override;

    virtual double eval(Eigen::Vector3d const &pos) const override;
};

} // namespace flow

#endif //WF0PS_H
