#!/usr/bin/env python3
# -*- coding: utf-8; -*-

import katoptron.CMAME.model as model

import katoptron as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katoptron.utilities as u


def evaluate_one_ensemble(ensemble_size, randomVariable, msh, comm, file_dir):

    work_dir = os.getcwd()
    args = parseargs()

    p = model.defaut_params
    p['MueLu xml from current_dir'] = False
    p['MueLu xml use template'] = False
    p['MueLu xml file name'] = '/../preconditioners/SIMPLE_direct.xml'

    p['Correlation length X'] = 5.
    p['Correlation length Y'] = 5.
    p['Number of KL Terms X'] = 15
    p['Number of KL Terms Y'] = 15
    p['Dispersion level'] = 0.3

    p['Write txt files'] = True
    p['Write matrix and vectors'] = True

    pm = model.plate_model(ensemble_size, randomVariable,
                           msh, comm, file_dir, work_dir, p)
    pm.run(args.k)


def evaluate_all_ensembles(ensemble_size, randomVariable, msh, comm, file_dir):

    directory = str(ensemble_size)
    if not os.path.exists(directory):
        os.makedirs(directory)
    os.chdir(directory)

    N = randomVariable.shape[0]
    N_ensemble = (N // ensemble_size)
    for ensemble_index in range(0, N_ensemble):
        directory = str(ensemble_index)
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)

        evaluate_one_ensemble(ensemble_size, randomVariable[ensemble_index:(
            ensemble_index+ensemble_size), :], msh, comm, file_dir)

        os.chdir('..')

    os.chdir('..')


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")
    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = u.fuse_meshes('plate_1.msh', 'plate_1_tmp', 'plate_2.msh',
                        'plate_2_tmp', 'plate_fused', file_dir, work_dir)

    np.random.seed(42)
    N = 32
    num_random_variables = 105
    randomVariable = np.random.normal(0, 1, num_random_variables*N)
    randomVariable = np.resize(randomVariable, (N, num_random_variables))

    np.savetxt('randomVariable.txt', randomVariable)

    evaluate_all_ensembles(1, randomVariable, msh, comm, file_dir)


if __name__ == "__main__":
    main()
