/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wCacheLine2.h"

using namespace tbox;

CacheLine2::CacheLine2(int _n) : Cache(_n), gauss(_n)
{
    // get number of gauss points
    size_t ngp = gauss.getN();

    // pre-compute dff and ff
    dff.resize(ngp);
    ff.resize(ngp);
    for (size_t k = 0; k < ngp; ++k)
    {
        Eigen::Vector3d const &ksi = gauss.getP(k);
        sf.evalD(ksi, dff[k]);
        sf.eval(ksi, ff[k]);
    }
}

Gauss &CacheLine2::getVGauss()
{
    return gauss;
}
