/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WHEX8_H
#define WHEX8_H

#include "tbox.h"
#include "wLazy.h"
#include "wElement.h"
#include "wCacheHex8.h"

namespace tbox
{

/**
 * @brief Linear hexaedron finite element
 * @authors Kim Liegeois, Romain Boman, Adrien Crovato
 */
class TBOX_API Hex8 : public Element
{
#ifndef SWIG
private:
    inline static CacheHex8 &getCache(int n);

protected:
    virtual void buildJ() override;
    virtual void computeV() override;
#endif

public:
    Hex8(int n, Tag *_ptag, Tag *_etag, std::vector<Tag *> &_parts, std::vector<Node *> &nods);
    virtual ~Hex8() {}
    virtual ELTYPE type() const override { return ELTYPE::HEX8; }

#ifndef SWIG
    virtual void initValues(bool isvol, int ordr = 2) override;
    virtual void initGradients() override;
    virtual void update() override;
    virtual Cache &getVCache() const override;
#endif
};

#include "wHex8.inl"

} // namespace tbox

#endif //WHEX8_H
