/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wSfTri3.h"

using namespace tbox;

void SfTri3::eval(Eigen::Vector3d const &ksi, Eigen::VectorXd &ff) const
{
    ff.resize(3);
    double x = ksi(0);
    double y = ksi(1);

    ff(0) = 1.0 - x - y;
    ff(1) = x;
    ff(2) = y;
}

void SfTri3::evalD(Eigen::Vector3d const &ksi, Eigen::MatrixXd &dff) const
{
    dff.resize(2, 3);

    dff(0, 0) = -1.0;
    dff(1, 0) = -1.0;
    dff(0, 1) = 1.0;
    dff(1, 1) = 0.0;
    dff(0, 2) = 0.0;
    dff(1, 2) = 1.0;
}
